@@ -26,6 +26,7 @@ from xml.parsers import expat
 
 import core, util
 from util import word_wrap, ProgressFactory, is_image_list
+from gamera.plugins import runlength
 from gamera.symbol_table import SymbolTable
 from config import config
 
@@ -99,7 +100,7 @@ class WriteXML:
       self._write_core(stream)
 
    def _write_core(self, stream, indent=0):
-      progress = util.ProgressFactory("Saving XML...", 1)
+      progress = util.ProgressFactory("Saving XML...", 1, numsteps=32)
       try:
          self._write_symbol_table(stream, self.symbol_table, indent=indent)
          if isinstance(self.glyphs, core.ImageBase):
@@ -250,7 +251,7 @@ class LoadXML:
       self._parser.StartElementHandler = self._start_element_handler
       self._parser.EndElementHandler = self._end_element_handler
       self._stream = stream
-      self._progress = util.ProgressFactory("Loading XML...", self._stream_length)
+      self._progress = util.ProgressFactory("Loading XML...", self._stream_length, numsteps=32)
       try:
          try:
             self._parser.ParseFile(stream)
@@ -435,7 +436,7 @@ class LoadXML:
       self._property_value.append(data)
 
 def glyphs_from_xml(filename, feature_functions = None):
-   """**glyphs_from_xml** (FileOpen *filename*, Features *feature_functions* = ``None``)
+   """**glyphs_from_xml** (*filename*, *feature_functions* = ``None``)
 
 Return a list of glyphs from a Gamera XML file."""
    glyphs = LoadXML().parse_filename(filename).glyphs
@@ -445,7 +446,7 @@ Return a list of glyphs from a Gamera XML file."""
    return glyphs
 
 def glyphs_with_features_from_xml(filename, feature_functions = None):
-   """**glyphs_with_features_from_xml** (FileOpen *filename*, Features *feature_functions* = ``None``)
+   """**glyphs_with_features_from_xml** (*filename*, *feature_functions* = ``None``)
 
 Loads glyphs from a Gamera XML file, and then generates features
 for all of those glyphs.  The set of features can be specified with the
@@ -456,7 +457,7 @@ for all of those glyphs.  The set of features can be specified with the
    return glyphs_from_xml(filename, feature_functions)
 
 def glyphs_to_xml(filename, glyphs, with_features=True):
-   """**glyphs_to_xml** (FileSave *filename*, *with_features* = ``True``)
+   """**glyphs_to_xml** (*filename*, *glyphs*, *with_features* = ``True``)
 
 Saves the given list of glyphs to a Gamera XML file.
 
@@ -556,7 +557,7 @@ class StripTag:
       self._output.close()
 
 def strip_features(input_filename, output_filename):
-   """**strip_features** (FileOpen *input_filename*, FileSave *output_filename*)
+   """**strip_features** (*input_filename*, *output_filename*)
 
 Strips the features from a Gamera XML file.  Provided mainly to reduce filesizes for
 files created before saving features was an option.
