@@ -502,7 +502,7 @@ class ClassifierFrame(ImageFrameBase):
       self.menu = None
       self.default_segmenter = -1
       self._save_by_criteria_dialog = [1] * 10
-      self._group_and_guess_dialog = [0, 4, 4, 16]
+      self._group_and_guess_dialog = [0, 4, 4, 16, 0]
 
       ImageFrameBase.__init__(
          self, parent, id,
@@ -644,8 +644,8 @@ class ClassifierFrame(ImageFrameBase):
          ("Guess all", self._OnGuessAll),
          ("&Guess selected", self._OnGuessSelected),
          (None, None),
-         ("Group and guess all", self._OnGroupAndGuessAll),
-         ("Group &and guess selected", self._OnGroupAndGuessSelected),
+         ("Group and guess all...", self._OnGroupAndGuessAll),
+         ("Group &and guess selected...", self._OnGroupAndGuessSelected),
          (None, None),
          ("Confirm all", self._OnConfirmAll),
          ("&Confirm selected", self._OnConfirmSelected),
@@ -1434,18 +1434,40 @@ class ClassifierFrame(ImageFrameBase):
       self.set_image(ccs, image, weak=False)
 
    def _OnSavePageCollectionAsImages(self, event):
-      self._OnSaveAsImages(self.multi_iw.id.GetAllItems())
+      list = self.multi_iw.id.GetAllItems()
+      if len(list) == 0:
+         gui_util.message("No glyphs in page.")
+         return
+      self._OnSaveAsImages(list)
 
    def _OnSaveSelectedAsImages(self, event):
-      self._OnSaveAsImages(self.get_all_selected()[0])
+      list = self.get_all_selected()[0]
+      if len(list) == 0:
+         gui_util.message("No glyphs selected.")
+         return
+      self._OnSaveAsImages(list)
 
    def _OnSaveClassifierCollectionAsImages(self, event):
-      self._OnSaveAsImages(self._classifier.get_glyphs())
+      list = self._classifier.get_glyphs()
+      if len(list) == 0:
+         gui_util.message("No glyphs in classifier.")
+         return
+      self._OnSaveAsImages(list)
 
    def _OnSaveAsImages(self, list):
-      filename = gui_util.directory_dialog(self._frame)
-      if filename:
-         classifier_stats.GlyphStats(list).write(filename)
+      dirname = gui_util.directory_dialog(self._frame)
+      if dirname:
+         glyphs = [g for g in list]
+         glyphs.sort(lambda g1,g2: cmp(g1.get_main_id().lower(),g2.get_main_id().lower()))
+         lastid = ""
+         nr = 0
+         for glyph in glyphs:
+            thisid = glyph.get_main_id().lower()
+            if thisid != lastid:
+               nr = 0
+               lastid = thisid
+            nr += 1
+            glyph.save_PNG(os.path.join(dirname, "%s-%03d.png" % (thisid,nr)))
 
    ########################################
    # CLASSIFIER MENU
@@ -1483,12 +1505,13 @@ class ClassifierFrame(ImageFrameBase):
          [Choice('Grouping function', ['Bounding Box', 'Shaped'], self._group_and_guess_dialog[0]),
           Int('Distance threshold', range=(0, 100), default=self._group_and_guess_dialog[1]),
           Int('Maximum number of parts per group', default=self._group_and_guess_dialog[2]),
-          Int('Maximum solveable subgraph size', default=self._group_and_guess_dialog[3])],
-         name = 'Save by criteria...')
+          Int('Maximum solveable subgraph size', default=self._group_and_guess_dialog[3]),
+          Choice('Grouping criterion', ['min', 'avg'], self._group_and_guess_dialog[4])],
+         name = 'Group and guess...')
       results = dialog.show(self._frame)
       if results is None:
          return
-      function, threshold, max_parts_per_group, max_graph_size = results
+      function, threshold, max_parts_per_group, max_graph_size, criterion = results
       self._group_and_guess_dialog = results
 
       if function == 0:
