@@ -260,7 +260,8 @@ def word_wrap(stream, l, indent=0, width=78):
          i = l.rfind(' ', p, p + width)
          if i == -1:
             stream.write(l[p:])
-         stream.write(l[p:i])
+         else:
+            stream.write(l[p:i])
       stream.write('\n')
       p = i + 1
 
@@ -328,9 +329,9 @@ class ProgressText:
             sys.stdout.write("\n")
             sys.stdout.flush()
 
-def ProgressFactory(message, length=1):
+def ProgressFactory(message, length=1, numsteps=0):
    if has_gui.gui != None:
-      return has_gui.gui.ProgressBox(message, length)
+      return has_gui.gui.ProgressBox(message, length, numsteps)
    elif config.get("progress_bar"):
       return ProgressText(message, length)
    else:
@@ -545,7 +546,7 @@ class CallbackSet(sets.Set, CallbackObject):
                yield i
       self.trigger_callback('remove', remove_iter())
       self.trigger_callback('add', add_iter())
-      self.Set.symmetric_difference_update(self, iterable)
+      sets.Set.symmetric_difference_update(self, iterable)
       self.trigger_callback('length_change', len(self))
 
    def intersection_update(self, iterable):
@@ -554,7 +555,7 @@ class CallbackSet(sets.Set, CallbackObject):
             if not i in iterable:
                yield i
       self.trigger_callback('remove', iter())
-      self.Set.intersection_update(self, iterable)
+      sets.Set.intersection_update(self, iterable)
       self.trigger_callback('length_change', len(self))
 
    def union_update(self, other):
