@@ -22,7 +22,7 @@
 
 #include "gamera.hpp"
 #include "image_utilities.hpp"
-#include "morphology.hpp"
+#include "misc_filters.hpp"
 #include <exception>
 #include <vector>
 #include <algorithm>
@@ -255,7 +255,7 @@ Image* tsai_moment_preserving_threshold(const T &m, int storage_format) {
 ________________________________________________________________
 
         bin_ab.c
-        $Id: threshold.hpp 962 2007-05-21 13:29:29Z cdalitz $
+        $Id: threshold.hpp 1191 2010-02-03 15:00:25Z cdalitz $
         Copyright 1990, Blab, UiO
         Image processing lab, Department of Informatics
         University of Oslo
